# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .experience_type import ExperienceType

__all__ = ["TypeListResponse"]

TypeListResponse: TypeAlias = List[ExperienceType]
