# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.



from .._models import BaseModel

__all__ = ["ExperienceInstance"]


class ExperienceInstance(BaseModel):
    experience_instance_id: str

    request_id: str
