# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .budget_tags import BudgetTags

__all__ = ["TagRemoveResponse"]

TagRemoveResponse: TypeAlias = List[BudgetTags]
