# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.



from ..._models import BaseModel

__all__ = ["ExperienceType"]


class ExperienceType(BaseModel):
    description: str

    experience_type_id: str

    name: str

    request_id: str
